==> Botan-1.1.3.tgz: Botan-1.1.3/src/x509.cpp <==
/*************************************************
* X.509 Source File                              *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/x509.h>
#include <botan/filters.h>
#include <botan/asn1.h>
#include <botan/pem.h>
#include <botan/pk_algs.h>
#include <memory>

namespace Botan {

/*************************************************
* Return the default NULL algorithm paramaters   *
*************************************************/
SecureVector<byte> X509_PublicKey::DER_encode_params() const
   {
   DER_Encoder encoder;
   DER::encode_null(encoder);
   return encoder.get_contents();
   }

/*************************************************
* Check to make sure the paramaters are NULL     *
*************************************************/
void X509_PublicKey::BER_decode_params(DataSource& source)
   {
   BER_Decoder decoder(source);
   BER::decode_null(decoder);
   }

namespace X509 {

namespace {

/*************************************************
* Extract the fields of a subjectPublicKeyInfo   *
*************************************************/
void X509_extract_info(DataSource& source, AlgorithmIdentifier& alg_id,
                       SecureVector<byte>& x509_key)
   {
   try
      {
      BER_Decoder decoder(source);
      BER_Decoder sequence = BER::get_subsequence(decoder);
      BER::decode(sequence, alg_id);
      BER::decode(sequence, x509_key, BIT_STRING);
      sequence.verify_end();
      }
   catch(Decoding_Error)
      {
      throw Decoding_Error("X.509 public key decoding failed");
      }
   }

/*************************************************
* Decode a BER encoded X.509 public key          *
*************************************************/
void X509_BER_decode(X509_PublicKey& key, DataSource& source)
   {
   AlgorithmIdentifier alg_id;
   SecureVector<byte> x509_key;

   X509_extract_info(source, alg_id, x509_key);

   if(!key.valid_oid(alg_id.oid))
      throw Decoding_Error("X.509: OID mismatch, got " +
                           alg_id.oid.as_string() + ", expected " +
                           key.get_oid().as_string());

   Pipe output;
   output.process_msg(alg_id.parameters);
   output.process_msg(x509_key);
   key.BER_decode_params(output);
   output.set_default_msg(1);
   key.BER_decode_pub(output);
   }

}

/*************************************************
* DER or PEM encode a X.509 public key           *
*************************************************/
void encode(const X509_PublicKey& key, Pipe& pipe, X509_Encoding encoding)
   {
   DER_Encoder encoder;
   AlgorithmIdentifier alg_id(key);

   encoder.start_sequence();
   DER::encode(encoder, alg_id);
   DER::encode(encoder, key.DER_encode_pub(), BIT_STRING);
   encoder.end_sequence();

   SecureVector<byte> der = encoder.get_contents();
   if(encoding == PEM)
      pipe.write(pem_encode(der, der.size(), "PUBLIC KEY"));
   else
      pipe.write(der);
   }

/*************************************************
* PEM encode a X.509 public key                  *
*************************************************/
std::string PEM_encode(const X509_PublicKey& key)
   {
   Pipe pem;
   pem.start_msg();
   encode(key, pem, PEM);
   pem.end_msg();
   return pem.read_all_as_string();
   }

/*************************************************
* Decode a BER or PEM encoded X.509 public key   *
*************************************************/
void decode(X509_PublicKey& key, DataSource& source, X509_Encoding encoding)
   {
   if(encoding == PEM)
      {
      Pipe ber;
      ber.process_msg(pem_decode_check_label(source, "PUBLIC KEY"));
      X509_BER_decode(key, ber);
      }
   else
      X509_BER_decode(key, source);
   }

/*************************************************
* Extract a public key and return it             *
*************************************************/
X509_PublicKey* load_key(DataSource& source, X509_Encoding encoding)
   {
   AlgorithmIdentifier alg_id;
   SecureVector<byte> x509_key;

   if(encoding == PEM)
      {
      Pipe ber;
      ber.process_msg(pem_decode_check_label(source, "PUBLIC KEY"));
      X509_extract_info(ber, alg_id, x509_key);
      }
   else
      X509_extract_info(source, alg_id, x509_key);

   const std::string alg_name = OIDS::identify_algorithm(alg_id.oid);
   if(alg_name == "")
      throw Decoding_Error("Unknown algorithm OID: " + alg_id.oid.as_string());

   std::auto_ptr<X509_PublicKey> key(get_public_key(alg_name));
   if(!key.get())
      throw Decoding_Error("Unknown PK algorithm/OID: " + alg_name + ", " +
                           alg_id.oid.as_string());

   Pipe output;
   output.process_msg(alg_id.parameters);
   output.process_msg(x509_key);
   key->BER_decode_params(output);
   output.set_default_msg(1);
   key->BER_decode_pub(output);

   return key.release();
   }

}

}

==> Botan-1.1.5.tgz: Botan-1.1.5/src/x509.cpp <==
/*************************************************
* X.509 Source File                              *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/x509.h>
#include <botan/filters.h>
#include <botan/asn1.h>
#include <botan/pem.h>
#include <botan/pk_algs.h>
#include <memory>

namespace Botan {

/*************************************************
* Return the default NULL algorithm parameters   *
*************************************************/
SecureVector<byte> X509_PublicKey::DER_encode_params() const
   {
   DER_Encoder encoder;
   DER::encode_null(encoder);
   return encoder.get_contents();
   }

/*************************************************
* Check to make sure the parameters are NULL     *
*************************************************/
void X509_PublicKey::BER_decode_params(DataSource& source)
   {
   BER_Decoder decoder(source);
   BER::decode_null(decoder);
   }

namespace X509 {

namespace {

/*************************************************
* Extract the fields of a subjectPublicKeyInfo   *
*************************************************/
void X509_extract_info(DataSource& source, AlgorithmIdentifier& alg_id,
                       SecureVector<byte>& x509_key)
   {
   try
      {
      BER_Decoder decoder(source);
      BER_Decoder sequence = BER::get_subsequence(decoder);
      BER::decode(sequence, alg_id);
      BER::decode(sequence, x509_key, BIT_STRING);
      sequence.verify_end();
      }
   catch(Decoding_Error)
      {
      throw Decoding_Error("X.509 public key decoding failed");
      }
   }

/*************************************************
* Decode a BER encoded X.509 public key          *
*************************************************/
void X509_BER_decode(X509_PublicKey& key, DataSource& source)
   {
   AlgorithmIdentifier alg_id;
   SecureVector<byte> x509_key;

   X509_extract_info(source, alg_id, x509_key);

   if(!key.valid_oid(alg_id.oid))
      throw Decoding_Error("X.509: OID mismatch, got " +
                           alg_id.oid.as_string() + ", expected " +
                           key.get_oid().as_string());

   Pipe output;
   output.process_msg(alg_id.parameters);
   output.process_msg(x509_key);
   key.BER_decode_params(output);
   output.set_default_msg(1);
   key.BER_decode_pub(output);
   }

}

/*************************************************
* DER or PEM encode a X.509 public key           *
*************************************************/
void encode(const X509_PublicKey& key, Pipe& pipe, X509_Encoding encoding)
   {
   DER_Encoder encoder;
   AlgorithmIdentifier alg_id(key);

   encoder.start_sequence();
   DER::encode(encoder, alg_id);
   DER::encode(encoder, key.DER_encode_pub(), BIT_STRING);
   encoder.end_sequence();

   SecureVector<byte> der = encoder.get_contents();
   if(encoding == PEM)
      pipe.write(pem_encode(der, der.size(), "PUBLIC KEY"));
   else
      pipe.write(der);
   }

/*************************************************
* PEM encode a X.509 public key                  *
*************************************************/
std::string PEM_encode(const X509_PublicKey& key)
   {
   Pipe pem;
   pem.start_msg();
   encode(key, pem, PEM);
   pem.end_msg();
   return pem.read_all_as_string();
   }

/*************************************************
* Decode a BER or PEM encoded X.509 public key   *
*************************************************/
void decode(X509_PublicKey& key, DataSource& source, X509_Encoding encoding)
   {
   if(encoding == PEM)
      {
      Pipe ber;
      ber.process_msg(pem_decode_check_label(source, "PUBLIC KEY"));
      X509_BER_decode(key, ber);
      }
   else
      X509_BER_decode(key, source);
   }

/*************************************************
* Extract a public key and return it             *
*************************************************/
X509_PublicKey* load_key(DataSource& source, X509_Encoding encoding)
   {
   AlgorithmIdentifier alg_id;
   SecureVector<byte> x509_key;

   if(encoding == PEM)
      {
      Pipe ber;
      ber.process_msg(pem_decode_check_label(source, "PUBLIC KEY"));
      X509_extract_info(ber, alg_id, x509_key);
      }
   else
      X509_extract_info(source, alg_id, x509_key);

   const std::string alg_name = OIDS::identify_algorithm(alg_id.oid);
   if(alg_name == "")
      throw Decoding_Error("Unknown algorithm OID: " + alg_id.oid.as_string());

   std::auto_ptr<X509_PublicKey> key(get_public_key(alg_name));
   if(!key.get())
      throw Decoding_Error("Unknown PK algorithm/OID: " + alg_name + ", " +
                           alg_id.oid.as_string());

   Pipe output;
   output.process_msg(alg_id.parameters);
   output.process_msg(x509_key);
   key->BER_decode_params(output);
   output.set_default_msg(1);
   key->BER_decode_pub(output);

   return key.release();
   }

}

}
